"""
   Plotting for 2D-arrays depending on two parameters +
   interface for DX.

   COPYRIGHT

     Copyright (c) 2011, Ralf Hambach. All rights reserved.
     Use of this source code is governed by a BSD-style license that 
     can be found in the LICENSE file.
"""
__version__ = "$Revision: 366 $"
__author__ = "Ralf Hambach"
__date__ = "$Date: 2011-11-10 12:34:46 +0100 (Thu, 10 Nov 2011) $"

import numpy as np
from SRcommon import *
import matplotlib.pylab as plt
from matplotlib.widgets import Slider, Button, RadioButtons

# output of 2D image
def imshow2D_python(img,Bcc):
  plt.imshow(plt.transpose(img),origin='lower',\
            extent=[Bcc.min[0],Bcc.max[0],Bcc.min[1],Bcc.max[1]]);

def imshow2D_asy(file,img):
  file.write("\n// image data follows \n");
  file.write("int XMAX=%d, YMAX=%d; \n"%img.shape);
  file.write("real[][] img = {");
  for x in range(img.shape[0]):
    file.write("{");
    img[x].tofile(file,sep=',',format='%5.3g');
    if(x<img.shape[0]-1):
      file.write("},");
    else:
      file.write("}};\n");


# output of 3D image
def imshow3D_python(SRden,E,Bcc,rprimd=None,T=None,title=None):
  '''usage:
  '''
  # create new figure
  fig=plt.figure();
  if title:  plt.title(title);
  
  # axes
  axcolor='lightgoldenrodyellow';
  axImage  = fig.add_subplot(111); fig.subplots_adjust(left=0.25, bottom=0.25);
  axEnergy = fig.add_axes([0.25, 0.1, 0.65, 0.03], axisbg=axcolor);
  axSlice  = fig.add_axes([0.25, 0.15, 0.65, 0.03], axisbg=axcolor);
  axStyle  = fig.add_axes([0.1,0.7,0.1,0.1]);
  axLattice= fig.add_axes([0.4, 0.025, 0.1, 0.04]);
  axCMap   = fig.add_axes([0.6, 0.025, 0.1, 0.04]);
  axReset  = fig.add_axes([0.8, 0.025, 0.1, 0.04]);
  
  # sliders + Buttons
  slEnergy = Slider(axEnergy, E.label[0] , 0, E.size      , valinit=0, valfmt='%d')
  slSlice  = Slider(axSlice,  Bcc.label[2], 0, Bcc.shape[2], valinit=0, valfmt='%d')
  rbStyle  = RadioButtons(axStyle,["2D raw","2D blur"]);
  bLattice = Button(axLattice,'Atoms'  , color=axcolor, hovercolor=0.975)
  bCMap    = Button(axCMap,   'Rescale', color=axcolor, hovercolor=0.975)
  bReset   = Button(axReset,  'Reset'  , color=axcolor, hovercolor=0.975)
  
  # extent of images differs from Bcc.extent() by the size of one pixel
  pixel = Bcc.extent()/Bcc.shape;  
  Bmin  = Bcc.min-pixel/2; Bmax = Bcc.max+pixel/2;
  extent= [Bmin[0],Bmax[0],Bmin[1],Bmax[1]];

  # style of the figure
  interpolation = ["nearest"];

  # add lattice 
  def AddLattice(event):
    if rprimd is not None:
      lattice2D_python(rprimd,T,Bcc,axImage)

  # extend colorbar to current min/max
  def UpdateCMap(event):
    # get data from last image and calculate min/max values
    # manually (autoscale only checks min/max from data, which
    # are unfortunately not updated)
    im=axImage.get_images()[-1]; data=im.get_array();
    vmin=np.min(data); vmax=np.max(data); vmax_abs=np.max(np.abs(data));
    im.set_clim(-vmax_abs,vmax_abs);
    print "Scaling: [%8.5g,\t%8.5g]"%(-vmax_abs,vmax_abs);

  # removing everything from the axes and create a new image
  # 
  # Note: we create the only AxisImage object here by calling imshow(),
  #  all other functions modify this object through its member functions
  #  WHY? because repeated imshow() calls will add further images to the
  #       Axes.images container and slow down the update of the figure
  def Reset(event):
    axImage.clear(); 
    axImage.imshow(np.transpose(img[0,:-1,:-1,0]),origin='lower',\
                   extent=extent, interpolation=interpolation[0],\
                   vmin=-max_abs_SRden,vmax=max_abs_SRden,cmap=plt.get_cmap('RdBu'));
    print "Scaling: [%8.5g,\t%8.5g]"%(-max_abs_SRden,max_abs_SRden);
    UpdateImage(None);
    #E = int(slEnergy.val);  Z = int(slSlice.val);
    #cset=axImage.contour(img[E,:,:,Z],np.linspace(min_SRden,max_SRden,10),linewidths=2);
    #axImage.clabel(cset,inline=True,fmt='%1.1f',fontsize=10);
    
  # how to update the figure
  def UpdateImage(val):
    iE = int(slEnergy.val);  iZ = int(slSlice.val);
    # change last image, do not create new one (test for multiple instances?)
    print(" # of Images in Figure: %d"%(len(axImage.get_images())));
    axImage.get_images()[-1].set_data(np.transpose(img[iE,:,:,iZ]));
    axImage.set_title("%s = %5.2f %s,   %s = %5.2f %s" % \
     (E.label[0],   E.point([iE]),                           E.unit[0],\
      Bcc.label[2], Bcc.point([0,0,iZ])[2],Bcc.unit[2]));
    plt.draw();

  # style of figure
  def ChangeStyle(label):
    if label=="2D raw":
      interpolation[0]="nearest";
      Reset(None);
    elif label=="2D blur":
      interpolation[0]=None;
      Reset(None);

  # define events
  slEnergy.on_changed(UpdateImage);
  slSlice.on_changed(UpdateImage);
  rbStyle.on_clicked(ChangeStyle);
  bLattice.on_clicked(AddLattice);
  bCMap.on_clicked(UpdateCMap);
  bReset.on_clicked(Reset);

  # transform SRden to 4D-cube
  img=SRden.flatten().reshape(E.size,*Bcc.shape);
  max_SRden=np.max(SRden); min_SRden=np.min(SRden); max_abs_SRden=np.max(np.abs(SRden));

  Reset(None);
  #plt.show();
  

def imshow3D_dx(file,img,Bcc):

  # calculate grid properties
  # NOTE: last index is Bcc.shape-1, i.e. Bcc.max = Bcc.min + step * (Bcc.shape-1) 
  step=Bcc.extent()/(np.array(Bcc.shape,float)-0.999999); # NUM=1 possible

  # write img data
  file.write("object 1 class array items %d data follows \n" % (Bcc.size));
  img.tofile(file,sep=' ',format='%8.5g');           # last index is fastest
  file.write("\n");
  
  # write rectangular grid dimensions [in Angstrom]
  file.write("attribute \"dep\" string \"positions\"\n");
  file.write("object 2 class gridpositions counts %d %d %d\n" % tuple(Bcc.shape));
  file.write(" origin %8.5g %8.5g %8.5g \n" % tuple(Bcc.min));
  file.write(" delta  %8.5g %8.5g %8.5g \n" % (step[0],0,0));
  file.write(" delta  %8.5g %8.5g %8.5g \n" % (0,step[1],0));
  file.write(" delta  %8.5g %8.5g %8.5g \n" % (0,0,step[2]));
  file.write("object 3 class gridconnections counts %d %d %d\n" % tuple(Bcc.shape));
  file.write(" attribute \"element type\" string \"cubes\"\n");
  file.write(" attribute \"ref\" string \"positions\"\n");

  # write variable names for DX
  file.write("object \"electron density\" class field\n");
  file.write(" component \"data\" 1\n");
  file.write(" component \"positions\" 2\n");
  file.write(" component \"connections\" 3\n");

             
# draw lattice

def ConstructSolid(rprimd,T,min,max):
  '''creates list of atoms from the primitive cell <rprimd>
     and the atomic basis <T> (in reduced coordinates)
     within the rectangle spanned by the points <min>
     and <max> (in cartesian coordinates).'''
  
  # construct rather large lattice with nearly 3times the cells needed
  aCells= 2*np.ceil((max-min)/np.diag(rprimd));
  R = np.mgrid[ -aCells[0]:aCells[0]+1:1,
                -aCells[1]:aCells[1]+1:1,
                -aCells[2]:aCells[2]+1:1 ].reshape(3,-1).T;

  # adding atomic basis to lattice
  atoms=[];
  for r in range(len(R)):
    for t in range(len(T)):
      Pcc=np.dot(T[t].pos+R[r],rprimd);#*0.529177; # in c.c. [Angstrom]
      
      # clip to DIM (with border of 0.1 Angstrom)
      if ( (all(min-0.1<Pcc)) and (all(Pcc<max+0.1)) ):
        atom=T[t].copy(); atom.pos=Pcc;
        atoms.append(atom);

  return np.array(atoms);

def lattice2D_asy(rprimd,file):
  # TODO ?
  file.write("\n// lattice data follows \n");
  file.write("");

  
def lattice2D_python(rprimd,T,Bcc,ax):
  '''Add atoms to axis ax'''
  atoms=ConstructSolid(rprimd,T,Bcc.min,Bcc.max);
  # project coordinates to image plane (perpendicular to electron path)
  ax.scatter([a.pos[0] for a in atoms],[a.pos[1] for a in atoms],\
          40,np.array([a.color for a in atoms]));

def lattice3D_dx(file,rprimd,T,Bcc):
  '''Interface for OpenDX.'''

  atoms=ConstructSolid(rprimd,T,Bcc.min,Bcc.max);
  file.write(" %d\n" % (len(atoms)));
  file.write(" structure generated from SRplot.py \n");
  for a in atoms:
    file.write("%s\n"%(a.print_xyz()));
