"""                                                             
     Plot XY-spectra using matplotlib

   VERSION

     $Id: dp2plot.py 367 2011-11-15 12:58:13Z hambach $

   USAGE

     import dp_mdf;
     spectra=dp_mdf.GetMDF("tests/9pol*.mdf");
     def get_legend(mdf):  return "qpol=%d"%(mdf.param['iqpol']);
     PyPlot(spectra,'dsf',legend=get_legend);

   MDF interface:
   
    member functions: get_E(), get_Reps(), get_Ieps(), get_eel(), get_dsf()

    calc. parameters: param[<key>]
      <key>:  'filename', 'quantity', 'lfe', 'kernel',
              'npwwfn', 'nbands', 'npwmat', 'emin', 'emax', 'broad',
              'lomo', 'nkbz', 'nkibz', 'dsf_fact'
              'iqpol', 'q_rc', 'q_cc', 'qred_rc', 'qred_cc', 'G_rc', 'G_cc' 
   
   COPYRIGHT

     Copyright (c) 2011, Ralf Hambach. All rights reserved.
     Use of this source code is governed by a BSD-style license that 
     can be found in the LICENSE file.
"""
__version__ = "$Revision: 367 $"
__author__ = "Ralf Hambach"
__date__ = "$Date: 2011-11-15 13:58:13 +0100 (Tue, 15 Nov 2011) $"

import matplotlib.pylab as plt;
import numpy as np;
import dp_mdf;

def PyPlot(spectra,quantity,legend=None,ax=None,x=None,y=None,verbose=1):
  """
  Plot spectra using matplotlib
   spectra... list of MDF-objects
   quantity.. what to plot: 'Ieps','Reps','eels','dsf','other'
   ax     ... (opt) axes object, default: current axes
   legend ... (opt) function legend(mdf) returns legend
   verbose... (opt) verbosity
 
  if quantity=='other', you can specify x and y values manually:  
   x      ... function x(mdf) returns x-values to plot
   y      ... function y(mdf) returns y-values to plot
  """
  if not ax:
    fig=plt.figure(); ax=fig.add_subplot(111);
  if not legend:
    legend=lambda(mdf): "";
  if not x:
    x=lambda(mdf):  mdf.get_E();
    ax.set_xlabel('Energy [eV]');

  if quantity=='Ieps':
    y=lambda(mdf):  mdf.get_Ieps();
    ax.set_ylabel('$\Im \epsilon (q,\omega)$ [a.u.]');   
  elif quantity=='Reps':
    y=lambda(mdf):  mdf.get_Reps();
    ax.set_ylabel('$\Re \epsilon (q,\omega)$ [a.u.]');   
  elif quantity=='eels':
    y=lambda(mdf):  mdf.get_eel();
    ax.set_ylabel('$-\Im \epsilon^{-1} (q,\omega)$ [a.u.]');     
  elif quantity=='dsf':
    y=lambda(mdf):  mdf.get_dsf()*1000;
    ax.set_ylabel('$S (q,\omega)$ [1/keV]');   
  elif quantity=='other':
    if y is None:
      raise ValueError('Function y(mdf) for plotting expected');
  else:
    raise ValueError("Parameter quantity not recognized. \n"+\
     " Should be one of 'Ieps','Reps','eels','dsf', or 'other'");

  i=0;
  for mdf in spectra:
    ax.plot(x(mdf),y(mdf),label=legend(mdf));
    if verbose>2:
      print " s%d: file=%s,"%(i, mdf.param['filename'])+\
            "   iqpol=%d,"%(mdf.param['iqpol'])+\
            "   q_rc=(%5.3f,%5.3f,%5.3f)"%tuple(mdf.param['q_rc']);
    i+=1;

  ax.legend();
  #plt.show();


### Self-Tests
if __name__ == '__main__':

  spectra=dp_mdf.GetMDF("tests/9pol*.mdf");
  def get_legend(mdf):  return "qpol=%d"%(mdf.param['iqpol']);
  PyPlot(spectra,'dsf',legend=get_legend);

  plt.show();
